use crate::cli::cli_exec_from_dir;
use crate::git::CheckoutOptions;
use crate::git::GitOptions;
use crate::{cli::cli_exec, helm::repository::Version};
use std::error::Error;
use std::fs::{self, rename};

use base64::{engine::general_purpose, Engine};
use serde::Deserialize;
use serde::Serialize;

use crate::git::Git;

use super::{chart::Chart, repository::RepositoryImpl};
// A struct that represents a git repo with a chart
#[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
pub struct GitRepo {
    pub url: String,
    #[serde(alias = "ref")]
    pub git_ref: String,
    pub path: String,
    #[serde(default = "default_git_bin")]
    pub(crate) git_bin: String,
}

fn default_git_bin() -> String {
    "git".to_string()
}

impl RepositoryImpl for GitRepo {
    fn pull_chart(&self, chart: Chart, workdir_path: String) -> Result<String, Box<dyn Error>> {
        let repo_name = general_purpose::STANDARD_NO_PAD.encode(self.get_url().clone());
        let git_instance = Git::new(self.get_url(), repo_name.clone());

        let git_opts = GitOptions::new(self.git_bin.clone(), Some(workdir_path.clone()));
        git_instance.clone(git_opts.clone())?;

        let checkout_opts = CheckoutOptions {
            create: true,
            git_ref: self.git_ref.clone(),
        };

        git_instance.checkout(git_opts, checkout_opts)?;

        let old_dir_name = format!(
            "{}/{}/{}/{}",
            workdir_path,
            repo_name,
            self.path,
            chart.name.clone()
        );

        let cmd = format!("helm show chart {}", old_dir_name);
        let helm_stdout = cli_exec(cmd)?;
        let new_dir_name: String;
        match serde_yaml::from_str::<Version>(&helm_stdout) {
            Ok(res) => {
                new_dir_name = format!("{}/{}-{}", workdir_path, chart.name.clone(), res.version);
                rename(old_dir_name, new_dir_name.clone())?;
            }
            Err(err) => return Err(Box::from(err)),
        };

        // Cleaning up
        fs::remove_dir_all(format!("{}/{}", workdir_path, repo_name))?;

        // Get the version
        let cmd = "helm show chart . | yq '.version'".to_string();
        let _version = cli_exec_from_dir(cmd, new_dir_name.clone())?;
        Ok(new_dir_name)
    }

    fn get_url(&self) -> String {
        self.url.clone()
    }

    fn get_version(&self, chart_path: String) -> Result<String, Box<dyn Error>> {
        let cmd = "helm show chart . | yq '.version'".to_string();
        let version = cli_exec_from_dir(cmd, chart_path.clone())?;
        Ok(version)
    }
}
